// largepi: Large integers used to compute pi
//   (to be linked with large.cpp, listed in Appendix A).


// Copyright(c) 1996 Leendert Ammeraal. All rights reserved.
// This program text is based on Chapter 2 and Appendix A of
//
//    Ammeraal, L. (1996) Algorithms and Data Structures in C++,
//       Chichester: John Wiley.

#include <strstrea.h>
// This filename strstrea.h is in accordance
// with the draft C++ standard. If your compiler 
// does not accept it, use strstream.h instead.

#include <fstream.h>
#include <time.h>
#include <stdlib.h>
#include "large.h"

// For the computation of pi:
large Atan(uint k, const large &N)
{  // Computes N * atan(1.0/k) as a large integer
   large a = 0,
      w = N * k,
      zero = 0,
      k2 = k * k,
      i = -1,
      two = 2;
   cout << "k = " << k << endl;
   while (w != zero)
   {  a += (w /= k2)/(i += two);
      a -= (w /= k2)/(i += two);
   }
   return a;
}

int main()
{  int n, m, FileOutput;
   cout << "Computation of pi. Number of decimals: ";
   cin >> n; m = n + 3;
   cout << "Copy of output to file pi.txt desired (y/n)? ";
   char answer;
   cin >> answer;
   FileOutput = (answer == 'Y' || answer == 'y');
   ofstream ofile;
   if (FileOutput) ofile.open("pi.txt");
   char *buf = new char[n + 5];
   ostrstream oo(buf, n + 5);
   large TenPower, Pi;
   clock_t tStart, tEnd;
   tStart = clock();

   TenPower = power(5, m); TenPower <<= m;
      // Faster than TenPower = power(10, m); 
   Pi = (Atan(18, TenPower * 48) + Atan(57, TenPower * 32)
         - Atan(239, TenPower * 20))/1000;
   cout << "Digits of pi:" << endl;
   oo << Pi;   // The result is now available in array buf.
   tEnd = clock();
      
   cout << buf[0] << '.';
   if (FileOutput) ofile << buf[0] << '.';
   for (int i=1; i<=n; i++)
   {  if (i % 50 == 1)
      {  cout << endl;
         if (FileOutput) ofile << endl;
      }  else
      if (i % 10 == 1)
      {  cout << " ";
         if (FileOutput) ofile << " ";
      }
      cout << buf[i];
      if (FileOutput) ofile << buf[i];
   }
   cout << "\nTime: " << (tEnd - tStart) << " ticks\n";
   return 0;
}
