// bmmsort.h: Balanced multiway merging sort.

// Copyright(c) 1996 Leendert Ammeraal. All rights reserved.
// This program text occurs in Chapter 3 of
//
//    Ammeraal, L. (1996) Algorithms and Data Structures in C++,
//       Chichester: John Wiley.

#include <assert.h>
#include <fstream.h>
#include <stdio.h>
#include <stdlib.h>
const int N=5;
static int in, out, inbuf[3], iOut, TapesWrittenTo[3];
static long nn, nRead[3];
static fstream t[6];

template <class T>
void WriteTape(int i, const T &x)
{  t[i].write((char *)&x, sizeof(T));
}

template <class T>
int ReadTape(int i, T &x)
{  t[i].read((char *)&x, sizeof(T));
   return !t[i].fail();
}

template <class T>
int oi(T *a, int i)  // Output and input of one item
{  WriteTape(out+iOut, a[i]);
   TapesWrittenTo[iOut] = 1;
   if (nRead[i] == nn) nRead[i] = inbuf[i] = 0; else
   {  inbuf[i] = ReadTape(in + i, a[i]);
      if (inbuf[i]) nRead[i]++;
   }
   return 1;
}

template <class T>
int oi(T *a, int i, int j)
{  return a[i] <= a[j] && oi(a, i);
}

template <class T>
int oi(T *a, int i, int j, int k)
{  return a[i] <= a[j] && a[i] <= a[k] && oi(a, i);
}

template <class T>
void InsertSort(T *a, int n)
{  int j;
   T x;
   for (int i=0; i<n; i++)
   {  x = a[i];
      for (j=i-1; j>=0; j--)
         if (x < a[j]) a[j+1] = a[j]; else break;
      a[j+1] = x;
   }
}

template <class T>
int BMMSort(const char *InpFileName, const char *OutFileName,
   const T &dummy) // dummy indicates item type for template
{  ifstream InpFile;
   ofstream OutFile;
   T a[N], x;
   InpFile.open(InpFileName, ios::in | ios::nocreate);
   if (!InpFile) return 1; // Cannot open input file
   int i, j, n, count, LastTape;
   char strTmp[6][L_tmpnam];
   for (i=0; i<6; i++)
   {  tmpnam(strTmp[i]);
      t[i].open(strTmp[i], ios::out | ios::trunc | ios::binary);
      // You may have to remove this part:  | ios::binary.
      // It is required with MS-DOS, however.
      // The same applies elsewhere in this program.
      if (t[i].fail())
      {  cout << "Cannot open temporary files\n"; exit(1);
      }
   }
   i = 0; // Write runs of n items to t0, t1 and t2:
   for (;;)
   {  for (n=0; n<N; n++) if (!(InpFile >> a[n])) break;
      InsertSort(a, n);
      for (j=0; j<n; j++) WriteTape(i, a[j]);
      i++; i %= 3;
      if (!InpFile) break;
   }
   nn = N; in = 0; out = 3;
   iOut = 0;
   for (i=0; i<3; i++)
   {  t[j=in+i].close();
      t[j].open(strTmp[j], ios::in | ios::binary);
   }
   for (;;)
   {  nRead[0] = nRead[1] = nRead[2] = 0;
      for (i=0; i<3; i++)
      {  inbuf[i] = ReadTape(in + i, a[i]);
         if (inbuf[i]) nRead[i]++;
      }
      while (inbuf[0] && inbuf[1] && inbuf[2])
         oi(a, 0, 1, 2) || oi(a, 1, 2, 0) || oi(a, 2, 0, 1);
      while (inbuf[0] && inbuf[1]) oi(a, 0, 1) || oi(a, 1, 0);
      while (inbuf[1] && inbuf[2]) oi(a, 1, 2) || oi(a, 2, 1);
      while (inbuf[2] && inbuf[0]) oi(a, 2, 0) || oi(a, 0, 2);
      while (inbuf[0]) oi(a, 0);
      while (inbuf[1]) oi(a, 1);
      while (inbuf[2]) oi(a, 2);
      if (t[in].fail() && t[in+1].fail() && t[in+2].fail())
      {  count = 0;
         for (i=0; i<3; i++)
         {  if (TapesWrittenTo[i]) {LastTape = out+i; count++;}
            TapesWrittenTo[i] = 0;
         }
         if (count <= 1) break;
         in = out; out = 3 - in; nn *= 3; iOut = 0;

         for (i=0; i<3; i++)
         {  t[j=in+i].close();
            t[j].open(strTmp[j], ios::in | ios::binary);
            t[j=out+i].close();
            t[j].open(strTmp[j], 
               ios::out | ios::trunc | ios::binary);
         }
      }  else if (++iOut == 3) iOut = 0;
   }
   InpFile.close();
   OutFile.open(OutFileName, ios::out);
   if (!OutFile) return 2; // Cannot open output file
   char *pSepar = (sizeof(T) == 1 ? "" : "\n");
   if (count)
   {  t[LastTape].close();
      t[LastTape].open(strTmp[LastTape], ios::in | ios::binary);
      while (ReadTape(LastTape, x))
         OutFile << x << pSepar;
   }  // else count == 0: empty input and output files
   OutFile.close();
   InpFile.close();
   for (i=0; i<6; i++) {t[i].close(); remove(strTmp[i]);}
   return 0;
}

