// qsorta: Function compatible with the qsort library function.

// Copyright(c) 1996 Leendert Ammeraal. All rights reserved.
// This program text occurs in Chapter 3 of
//
//    Ammeraal, L. (1996) Algorithms and Data Structures in C++,
//       Chichester: John Wiley.

#include <stdlib.h>

static void swap(void *p, void *q, size_t width)
{  char w, *pp=(char*)p, *qq=(char*)q;
   while (width--)
   {  w = *pp;
      *pp++ = *qq;
      *qq++ = w;
   }
}
static void swapint(void *p, void *q, size_t width)
{  register int w, *pp=(int*)p, *qq=(int*)q;
   while (width--)
   {  w = *pp;
      *pp++ = *qq;
      *qq++ = w;
   }
}

static void swap3(void *p, void *q, void *r, size_t width)
{  char w, *pp=(char*)p, *qq=(char*)q, *rr=(char*)r;
   while (width--)
   {  w = *pp;
      *pp++ = *qq;
      *qq++ = *rr;
      *rr++ = w;
   }
}

static void swap3int(void *p, void *q, void *r, size_t width)
{  register int w, *pp=(int*)p, *qq=(int*)q, *rr=(int*)r;
   while (width--)
   {  w = *pp;
      *pp++ = *qq;
      *qq++ = *rr;
      *rr++ = w;
   }
}

static void (*swp)(void *p, void *q, size_t width);
static void (*swp3)(void *p, void *q, void *r, size_t width);
static size_t width1;
static int (*fcmp)(const void *, const void *);

static void qsort1(char *a, size_t n, size_t width)
{  int ci, cj;
   size_t nleft, nright;
   char *pi, *pj, *ph, *plast;
   while (n >= 3)
   {  plast = a + (n - 1) * width;
      ph = a + ((n-1)/2) * width;
      if (fcmp(a, ph) > 0)
      {  if (fcmp(a, plast) > 0)
         {  if (fcmp(ph, plast) > 0) swp(a, plast, width1);//321
            else swp3(a, ph, plast, width1);               //312
         }  else swp(a, ph, width1);                       //213
      }  else
      if (fcmp(a, plast) > 0) swp3(a, plast, ph, width1);  //231
      else
      if (fcmp(ph, plast) > 0) swp(ph, plast, width1);     //132
      // else do nothing                                   //123

      if (n == 3) return;
      pi = a + width; pj = plast - width;
      // Pivot a[ph] is median of three elements
      do
      {  while ((ci = fcmp(pi, ph)) < 0) pi += width;
         while ((cj = fcmp(pj, ph)) > 0) pj -= width;
         if (pi < pj)
         {  swp(pi, pj, width1);
            if (ph == pi) ph = pj; else
            if (ph == pj) ph = pi;
         }  else
         {  if (pi == pj) {pi += width; pj -= width;} else
            {  if (ci == 0) pi += width;
               if (cj == 0) pj -= width;
            }
            break;
         }
      }  while ((pi += width) <= (pj -= width));
      nleft = (pj + width - a)/width;
      nright = (plast + width - pi)/width;
      if (nleft < nright)
      {  if (nleft > 1) 
            qsort1(a, nleft, width);
         a = pi; n = nright;
      }  else
      {  if (nright > 1)
            qsort1(pi, nright, width);
         n = nleft;
      }
   }
   if (n == 2 && fcmp(a, a + width) > 0) 
      swp(a, a + width, width1);
}

void qsorta(void *base, size_t n, size_t width,
   int (*cmp)(const void *, const void *))
{  if (width % sizeof(int) == 0)
   {  swp = swapint; swp3 = swap3int; 
      width1 = width/sizeof(int);
   }  else
   {  swp = swap; swp3 = swap3; width1 = width;
   }
   fcmp = cmp;
   qsort1((char *)base, n, width);
}

