// alnklist: A linked list implemented in an array.

// Copyright(c) 1996 Leendert Ammeraal. All rights reserved.
// This program text occurs in Chapter 4 of
//
//    Ammeraal, L. (1996) Algorithms and Data Structures in C++,
//       Chichester: John Wiley.

#include <iostream.h>
#include <stdlib.h>
#include <ctype.h>
struct elem{int info; int next;};

class alnklist {
public:
   alnklist();
   elem *NewNode();          // Instead of new operator
   void DeleteNode(elem *p); // Instead of delete operator
   void iteratorStart(elem *p=NULL)
   {  iCur = (p ? int(p - a) : iStart);
   }
   elem *iterator();
   void PrintList();
   elem *FindPosition(int x, int &found);
   void InsertBefore(elem *p, int x);
   void Remove(elem *p);
private:
   enum {len = 10};
   elem a[len];
   int iFreeStart, iStart, iEnd, iCur;
};
alnklist::alnklist()
{  iFreeStart = 0;
   for (int i=1; i<len; i++) a[i-1].next = i;
   a[len-1].next = -1;
   iStart = iEnd = int(NewNode() - a);
   a[iEnd].next = -1;
}

elem *alnklist::NewNode()
{  int i = iFreeStart;
   if (i >= 0) iFreeStart = a[iFreeStart].next; else
   {  cout << "Memory problem: array exhausted\n";
      exit(1);
   }
   return a + i;
}

void alnklist::DeleteNode(elem *p)
{  int i = int(p - a);
   a[i].next = iFreeStart;
   iFreeStart = i;
}

elem *alnklist::iterator()
{  if (iCur == iEnd) return NULL;
   int i = iCur;
   iCur = a[iCur].next;
   return a + i;
}

void alnklist::PrintList()
{  elem *p;
   iteratorStart();
   while ((p = iterator()) != NULL)
      cout << p->info << " ";
   cout << endl;
}

elem *alnklist::FindPosition(int x, int &found)
{  a[iEnd].info = x;
   int i = iStart;
   while (a[i].info < x) i = a[i].next;
   found = (i != iEnd && a[i].info == x);
   return a + i;
}

void alnklist::InsertBefore(elem *p, int x)
{  int i = int(p - a), j = int(NewNode() - a);
   a[j] = a[i];
   if (i == iEnd) iEnd = j;
   a[i].info = x;
   a[i].next = j;
}
void alnklist::Remove(elem *p)
{  int i = int(p - a), j = a[i].next;
   a[i] = a[j];
   if (j == iEnd) iEnd = i;
   DeleteNode(a + j);
}

int main()
{  alnklist L;
   elem *p;
   int x, found;
   char cmd;
   for (;;)
   {  cout << "Enter an integer, followed by I for insertion\n"
              "or by D for deletion, or enter Q to quit: ";
      if (!(cin >> x >> cmd)) break;
      cmd = toupper(cmd);
      p = L.FindPosition(x, found);
      if (cmd == 'I') L.InsertBefore(p, x); else
      if (cmd == 'D')
      {  if (found) L.Remove(p); else cout << "Not found.\n";
      }
      cout << "Contents: "; L.PrintList(); cout << endl;
   }
   return 0;
}
