// dbllist: Ordered circular doubly-linked list.

// Copyright(c) 1996 Leendert Ammeraal. All rights reserved.
// This program text occurs in Chapter 4 of
//
//    Ammeraal, L. (1996) Algorithms and Data Structures in C++,
//       Chichester: John Wiley.

#include <iostream.h>
#include <iomanip.h>
#include <stdlib.h>
#include <string.h>

struct elem{char *info; elem *prev, *next;};

class dbllist {
public:
   dbllist()
   {  pStart = new elem;  // Create sentinel
      pStart->prev = pStart->next = pStart;
      pStart->info = 0;
   }
   ~dbllist();

   void iteratorStart(elem *p=NULL)
   {  pCur = pStart;
   }

   elem *iteratorForward();
   elem *iteratorBackward();

   void PrintList();
   void append(const char *s);
   elem *FindPosition(const char *s);
   void remove(elem *p);

   dbllist(const dbllist& s)
   {  cout << "Copying not allowed.\n"; exit(1);
   }

   dbllist &operator=(const dbllist &s)
   {  cout << "Assignment not allowed.\n"; exit(1);
      return *this;
   }
private:
   elem *pStart, *pCur;
};

dbllist::~dbllist()
{  elem *p;
   iteratorStart();
   for (;;)
   {  p = iteratorForward();
      if (!p) break;
      delete p->info;
   }
   delete pStart; // Sentinel
}

elem *dbllist::iteratorForward()
{  pCur = pCur->next;
   return pCur == pStart ? NULL : pCur;
}

elem *dbllist::iteratorBackward()
{  pCur = pCur->prev;
   return pCur == pStart ? NULL : pCur;
}

void dbllist::PrintList()
{  elem *p;
   iteratorStart();
   while ((p = iteratorForward()) != NULL)
      cout << p->info << " ";
   cout << endl;
}
void dbllist::append(const char *s)
{  elem *p = new elem;
   int len = strlen(s);
   p->info = new char[len + 1];
   strcpy(p->info, s);
   elem *pEnd = pStart->prev;
   pEnd->next = p;
   p->prev = pEnd;
   p->next = pStart;
   pStart->prev = p;
}

elem *dbllist::FindPosition(const char *s)
{  iteratorStart();
   elem *p;
   do
      p = iteratorForward();
   while (p  &&  strcmp(p->info, s) != 0);
   return p;
}

void dbllist::remove(elem *p)
{  p->prev->next = p->next;
   p->next->prev = p->prev;
   delete p->info;
   delete p;
}

int main()
{  dbllist L;
   char buf[100];
   L.append("Alan"); L.append("Charles");
   L.append("Donald"); L.append("Erica");
   L.append("John"); L.append("Mary");
   cout << "Contents: "; L.PrintList();
   cout << "Name to be deleted: ";
   cin >> setw(100) >> buf;
   elem *p = L.FindPosition(buf);
   if (p)
   {  L.remove(p);
      cout << "Contents: "; L.PrintList();
   }  else cout << "Not found.\n";
   return 0;
}
