// lmsort: Merge sort applied to a linked list.

// Copyright(c) 1996 Leendert Ammeraal. All rights reserved.
// This program text occurs in Chapter 4 of
//
//    Ammeraal, L. (1996) Algorithms and Data Structures in C++,
//       Chichester: John Wiley.

#include <iostream.h>
#include <stdlib.h>
struct elem{int info; struct elem *next;};

class SList {
public:
   SList(){pStart = pEnd = NULL;}
   ~SList();
   void append(int i);
   void AppendFirstNode(SList &L);
   void iteratorStart(elem *p=NULL){pCur = (p ? p : pStart);}
   elem *iterator();
   void MergeSort();
   void PrintList();
   SList(const SList& s)
   {  cout << "Copying not allowed.\n"; exit(1);
   }
   SList &operator=(const SList &s)
   {  cout << "Assignment not allowed.\n"; exit(1);
      return *this;
   }
private:
   elem *pStart, *pEnd, *pCur;
};

SList::~SList()
{  while (pStart)
   {  pCur = pStart; pStart = pStart->next; delete pCur;
   }
}

void SList::append(int i)
{  pCur = pEnd;
   pEnd = new elem;
   pEnd->info = i;
   pEnd->next = NULL;
   (pCur ? pCur->next : pStart) = pEnd;
}

elem *SList::iterator()
{  elem *p = pCur;
   if (pCur) pCur = pCur->next;
   return p;
}

void SList::PrintList()
{  elem *p;
   iteratorStart();
   while ((p = iterator()) != NULL)
      cout << p->info << " ";
   cout << endl;
}

void SList::AppendFirstNode(SList &L)
// Move first node of list *this to the end of list L
// (L may be empty; *this is not):
{  // Detach first node of list *this:
   elem *p = pStart;
   pStart = pStart->next;
   p->next = NULL;
   // Attach that node (= *p) to list L:
   (L.pStart ? L.pEnd->next : L.pStart) = p;
   L.pEnd = p;
}

void SList::MergeSort()
{  SList L1, L2;
   if (pStart == NULL || pStart->next == NULL) return;
   iteratorStart();
   // Split *this into L1 and L2:
   while (pStart)
   {  AppendFirstNode(L1);
      if (pStart) AppendFirstNode(L2);
   }
   // Sort L1 and L2:
   L1.MergeSort(); L2.MergeSort();
    // Merge L1 and L2 into *this:
   while (L1.pStart && L2.pStart)
   {  if (L1.pStart->info < L2.pStart->info)
         L1.AppendFirstNode(*this); else
         L2.AppendFirstNode(*this);
   }
   // One of the following loops is empty:
   while (L1.pStart) L1.AppendFirstNode(*this);
   while (L2.pStart) L2.AppendFirstNode(*this);
}

int main()
{  int x;
   SList L;
   cout << "Enter some integers, followed by\n"
           "a nonnumeric character:\n";
   while (cin >> x) L.append(x);
   L.MergeSort();
   cout << "List contents after sorting:\n"; L.PrintList();
   return 0;
}
