// olist.h: Ordered linked list with sentinel.

// Copyright(c) 1996 Leendert Ammeraal. All rights reserved.
// This program text occurs in Chapter 4 of
//
//    Ammeraal, L. (1996) Algorithms and Data Structures in C++,
//       Chichester: John Wiley.

#include <stdlib.h>

template <class T>
struct elem{T info; elem *next;};

template <class T>
class olist {
public:
   olist() {pStart = pEnd = new elem<T>; pEnd->next = NULL;}
   olist(const olist& s);
   olist &operator=(const olist &s);
   virtual ~olist();
   void iteratorStart(elem<T> *p=NULL){pCur = (p ? p : pStart);}
   elem<T> *iterator();
   void PrintList();
   elem<T> *FindPosition(T x, int &found);
   void InsertBefore(elem<T> *p, T x);
   void Remove(elem<T> *p);
protected:
   elem<T> *pStart, *pEnd, *pCur;
};

template<class T>
void DelList(elem<T>* &start)
{  elem<T> *p;
   while (start->next != NULL)
   {  p = start; start = start->next; delete p;
   }
}

template <class T>
olist<T>::~olist()
{  DelList(pStart);  // Leave sentinel
   delete pEnd;      // Delete sentinel, too.
}

template <class T>
void CopyList(elem<T>* pSource, elem<T>* pStart, elem<T>* &pEnd)
{  while (pSource->next != NULL)
   {  *pEnd = *pSource;                 // Copy into sentinel
      pEnd = pEnd->next = new elem<T>;  // New sentinel
      pEnd->next = NULL;
      pSource = pSource->next;
   }
}

template <class T>
olist<T>::olist(const olist<T> &s)
{  pStart = pEnd = new elem<T>; pEnd->next = NULL; // sentinel
   CopyList(s.pStart, pStart, pEnd);
}

template <class T>
olist<T> &olist<T>::operator=(const olist &s)
{  if (&s != this)
   {  DelList(pStart);
      CopyList(s.pStart, pStart, pEnd);
   }
   return *this;
}

template <class T>
elem<T> *olist<T>::iterator()
{  if (pCur == pEnd) return NULL;
   elem<T> *p = pCur;
   pCur = pCur->next;
   return p;
}

template <class T>
void olist<T>::PrintList()
{  elem<T> *p;
   iteratorStart();
   while ((p = iterator()) != NULL) cout << p->info << " ";
   cout << endl;
}

template <class T>
elem<T> *olist<T>::FindPosition(T x, int &found)
{  pEnd->info = x;
   elem<T> *p = pStart;
   while (p->info < x) p = p->next;
   found = (p != pEnd && p->info == x);
   return p;
}

template <class T>
void olist<T>::InsertBefore(elem<T> *p, T x)
{  elem<T> *q = new elem<T>;
   *q = *p;
   if (p == pEnd) pEnd = q;
   p->info = x;
   p->next = q;
}

template <class T>
void olist<T>::Remove(elem<T> *p)
{  elem<T> *q = p->next;
   *p = *q;
   if (q == pEnd) pEnd = p;
   delete q;
}
