// hash1.cpp: Hashing based on linked lists (chaining).
//    (Implementation file, to be linked with hash1ap.cpp
//    or any other application file).

// Copyright(c) 1996 Leendert Ammeraal. All rights reserved.
// This program text occurs in Chapter 5 of
//
//    Ammeraal, L. (1996) Algorithms and Data Structures in C++,
//       Chichester: John Wiley.

#include <string.h>
#include "hash1.h"

list::~list()
{  elem *p = pStart, *q;
   while (p)
   {  q = p;
      p = p->next;
      delete[] q->name;
      delete q;
   }
}

void list::RemoveNode(const char *s)
{  elem *p = pStart, *q;
   if (p == NULL) return;
   if (strcmp(p->name, s) == 0)
   {  pStart = p->next;
      delete[] p->name;
      delete p;
      return;
   }
   for (;;)
   {  q = p->next;
      if (q == NULL) break;
      if (strcmp(q->name, s) == 0)
      {  p->next = q->next;
         delete[] q->name;
         delete q;
         return;
      }
      p = q;
   }
}

void list::ListInsert(const char *s, unsigned num)
{  elem *p = new elem;
   int len = strlen(s);
   p->name = new char[len + 1];
   strcpy(p->name, s);
   p->num = num;
   p->next = pStart;
   pStart = p;
}

elem *list::FindPosition(const char *s)const
{  elem *p = pStart;
   while (p && strcmp(p->name, s) != 0) p = p->next;
   return p;
}

unsigned StringHash::hash(const char *s)const
{  unsigned sum = 0;
   for (int i=0; s[i]; i++) sum += (i + 1) * s[i];
   return sum % N;
}

