// search.cpp: Implementation file.

// Copyright(c) 1996 Leendert Ammeraal. All rights reserved.
// This program text occurs in Chapter 5 of
//
//    Ammeraal, L. (1996) Algorithms and Data Structures in C++,
//       Chichester: John Wiley.

#include <string.h>
#include "search.h"

inline int max(int x, int y){return x > y ? x : y;}

search::search(const char *pat)
{  pattern = pat;
   m = strlen(pattern);
   skip2 = new int[m-1];
   const char *p;
   int ch, i, h, suflen;
   for (ch=0; ch<256; ch++)
   {  p = strrchr(pattern, ch);
      h = int(p ? p - pattern : -1);
      // If h != -1, pattern[h] == ch
      skip1[ch] = m - 1 - h;
   }
   for (i=m-2; i>=0; i--)
   {  p = pattern + i + 1; // Good suffix p follows position i.
      suflen = m - 1 - i;
      for (h=i; h>=0; h--)
         if (strncmp(pattern + h, p, suflen) == 0) break;
      skip2[i] = i + 1 - h; // h = -1 if not found.
   }
   // Due to the mismatch pattern[i] != text[j] = ch, the 
   // pattern will be shifted either skip1[ch]-(m-1-i) or 
   // skip2[i] positions to the right, whichever is larger.
}

char *search::find(char *text)
{  int last = m - 1, k = last, j, i;
   // pattern[last] will be compared with text[k]:
   char ch;
   int n = strlen(text);
   while (k < n)
   {  ch = text[k];
      if (pattern[last] != ch) k += skip1[ch]; else
      {  i = last; j = k;
         do
         {  if (--i < 0) return text + k - last; // Match found!
         }  while (pattern[i] == text[--j]);
         k += max(skip1[text[j]] - (last - i), skip2[i]);
      }
   }
   return 0;
}

