// Copyright(c) 1996 Leendert Ammeraal. All rights reserved.
// This program text occurs in Chapter 6 of
//
//    Ammeraal, L. (1996) Algorithms and Data Structures in C++,
//       Chichester: John Wiley.

// delnode: Deleting a single node in a binary search tree
#include <iostream.h>
#include <iomanip.h>

struct node {int info; node *pLeft, *pRight;};

class tree {
public:
   tree();
   void print()const{pr(root, 0);}
   node* &search(int x);
   int DelNode(int x);
private:
   node *root;
   void AddNode(int x, node* &p);
   void pr(const node *p, int nSpace)const;
};

tree::tree()
{  root = NULL;
   int x;
   char ch;
   while (cin >> x, !cin.fail()) AddNode(x, root);
   cin.clear();
   cin.get(ch);
}

void tree::AddNode(int x, node* &p)
{  if (p == NULL)
   {  p = new node;
      p->info = x;
      p->pLeft = p->pRight = NULL;
   }  else AddNode(x, x < p->info ? p->pLeft : p->pRight);
}

void tree::pr(const node *p, int nSpace)const
{  if (p)
   {  pr(p->pRight, nSpace += 6);
      cout << setw(nSpace) << p->info << endl;
      pr(p->pLeft, nSpace);
   }
}

node* &tree::search(int x)
{  node **p = &root;
   for(;;)
   {  if (*p == NULL) return *p;
      if (x < (*p)->info) p = &(*p)->pLeft; else
      if (x > (*p)->info) p = &(*p)->pRight; else return *p;
   }
}

int tree::DelNode(int x)
{  node* &p = search(x), *p0 = p, **qq, *q;
   if (p == NULL) return 0; // Not found
   if (p->pRight == NULL){p = p->pLeft; delete p0;} else
   if (p->pLeft == NULL){p = p->pRight; delete p0;} else
   {  // Node *p has a left and a right child
      qq = & p->pLeft;
      while ((*qq)->pRight) qq = & (*qq)->pRight;
      // qq contains the address of the pointer field
      // that points to a node without a right child.
      p->info = (*qq)->info;
      // Data copied from **qq to *p
      q = *qq;
      *qq = q->pLeft;
      // *qq replaced with left link field of **qq.
      delete q;
   }
   return 1; // Node deleted
}

int main()
{  cout <<
      "Enter some integers to be placed in a binary tree,\n"
      "followed by /:\n";
   tree t;
   cout << "Resulting tree, printed sideways:\n\n";
   t.print();
   cout << "\nEnter the value of the node to be deleted: ";
   int x;
   cin >> x;
   if (t.DelNode(x))
   {  cout << "Tree after deleting " << x << ":\n\n";
      t.print();
   }  else cout << "Node not found.\n";
   return 0;
}

