// Copyright(c) 1996 Leendert Ammeraal. All rights reserved.
// This program text occurs in Chapter 8 of
//
//    Ammeraal, L. (1996) Algorithms and Data Structures in C++,
//       Chichester: John Wiley.

// huffman.cpp: To be linked with huffman1 or huffman2
#include "huffman.h"
#include "priorq.h"

void GetStreams(ifstream &ifile, ofstream &ofile)
{  char iname[50], oname[50];
   cout << "Name of input file:  ";
   cin >> setw(50) >> iname;
   ifile.open(iname, ios::in | ios::nocreate | ios::binary);
      // ios::binary required with MSDOS, but possibly
      // not accepted with other environments.
   if (ifile.fail()) error("Cannot open input file.");
   cout << "Name of output file: ";
   cin >> setw(50) >> oname;
   ofile.open(oname, ios::out | ios::trunc | ios::binary); 
      // See above note.
   if (ofile.fail()) error("Cannot open output file.");
}

struct item
{  ulong freq;
   int ptr;
   item(){}
   item(ulong f, int p): freq(f), ptr(p){}
   int operator>(item &x){return freq < x.freq;}
   // u > v defined as u.freq < v.freq because we want
   // the items to appear in ascending (not in descending)
   // order of frequency.
};

HuffmanTree::HuffmanTree(const ulong *freq)
{  A = new node[511];
      // At most 256 leaves and 511 = 2 * 256 - 1 nodes,
      // see Exercise 6.12.
   n = 0;
   PrQ<item> Q(256);  // Size of priority queue
   for (int j=0; j<256; j++)
      if (freq[j]) Q.insert(item(freq[j], NewNode(nil, j)));
   item t1, t2;
   if (Q.empty()) error("Empty input file."); else
   {  for (;;)
      {  t1 = Q.remove();
         if (Q.empty()) break;
         t2 = Q.remove();
         Q.insert(item(t1.freq + t2.freq,
            NewNode(t1.ptr, t2.ptr)));
      }
      root = t1.ptr;
   }
}
