// Copyright(c) 1996 Leendert Ammeraal. All rights reserved.
// This program text occurs in Chapter 8 of
//
//    Ammeraal, L. (1996) Algorithms and Data Structures in C++,
//       Chichester: John Wiley.

// huffman2: File decompression (to be linked with
//           huffman.cpp)
#include "huffman.h"

void ReadFreqTable(ifstream &ifile, ulong *freq)
{  uchar jLower, jUpper, ch, ControlByte, LenCode;
   ushort u;
   ulong f;
   int j, j0;
   for (j=0; j<256; j++) freq[j] = 0;
   ifile.get(jLower);
   ifile.get(jUpper);
   for (j = jLower; j <= jUpper; )
   {  j0 = j;
      ifile.get(ControlByte);
      LenCode = ControlByte >> 6;
      j = j0 + (ControlByte & 0x3F);
      while (j0 < j)
      {  switch(LenCode) {
         case 0: f = 0; break;
         case 1: ifile.get(ch); f = ch; break;
         case 2: ifile.read((char*)&u, sizeof(ushort));
                 f = u; break;
         case 3: ifile.read((char*)&f, sizeof(ulong)); break;
         }
         freq[j0++] = f;
      }
   }
}

int main()
{  ifstream ifile;
   ofstream ofile;
   cout << "Huffman file decompression.\n";
   GetStreams(ifile, ofile);
   ulong freq[256];
   ReadFreqTable(ifile, freq);
   HuffmanTree Tr(freq);
   int root = Tr.n - 1, p = root, b, nBits;
   unsigned char buf1, buf2, buf3;
   ifile.get(buf1); ifile.get(buf2);
   for (;;)
   {  ifile.get(buf3);
      nBits = (ifile.fail() ? buf2 : 8);
      for (int i=0; i<nBits; i++)
      {  b = (buf1 >> 7) & 1;
         buf1 <<= 1;
         p = (b ? Tr.A[p].RightOrCh : Tr.A[p].left);
         if (Tr.A[p].left == nil)
         {  ofile.put(char(Tr.A[p].RightOrCh));
            p = root;
         }
      }
      if (ifile.fail()) break;
      buf1 = buf2; buf2 = buf3;
   }
   return 0;
}

