// Copyright(c) 1996 Leendert Ammeraal. All rights reserved.
// This program text occurs in Chapter 9 of
//
//    Ammeraal, L. (1996) Algorithms and Data Structures in C++,
//       Chichester: John Wiley.

// topol: Topological sorting; any cycles will be detected.
#include <fstream.h>
#include <iomanip.h>
#include <stdlib.h>

struct edge{int num; edge *next;};

class graph {
public:
   int min, n; // Minimum vertex number and n = max - min + 1
   void SetRange(const char *fname);
   void build(const char *fname);
   void StartNodes();
   void topol();
private:
   struct vertex{int count; edge *link;} *a;
   enum {nil=-1, absent=-2};
   int start; // Start of simulated linked list
              // for nodes without a predecessor
};

void graph::SetRange(const char *fname)
{  ifstream file(fname, ios::in | ios::nocreate);
   if (file.fail())
   {  cout << "Cannot open input file.\n";
      exit(1);
   }
   int max, x, first=1;
   for (;;)
   {  file >> x;
      if (file.fail()) break;
      if (first){min = max = x; first = 0;} else
      if (x < min) min = x; else
      if (x > max) max = x;
   }
   file.close();
   n = max - min + 1;
}

void graph::build(const char *fname)
{  int i, j, I, J;
   edge *p;
   a = new vertex[n];
   for (i=0; i<n; i++) {a[i].count = absent; a[i].link = 0;}
   ifstream file(fname, ios::in | ios::nocreate);
   for (;;)
   {  file >> I >> J;
      if (file.fail()) break;
      i = I - min; j = J - min;
      if (a[i].count == absent) a[i].count = 0;
      if (a[j].count == absent) a[j].count = 0;
      p = new edge;
      p->num = j;
      p->next = a[i].link;
      a[i].link = p;
      a[j].count++;
   }
   file.close();
}

void graph::StartNodes()
{  int i;
   // Set up simulated linked list of start nodes:
   start = nil;
   for (i=0; i<n; i++)
      if (a[i].count == 0){a[i].count = start; start = i;}
}

void graph::topol()
{  int i, j, J, k;
   edge *p;
   cout << "Topological order:\n";
   for (i=0; i<n; i++)
   {  while (a[i].count == absent) i++;
      if (start == nil)
      {  cout << "There is a cycle.\n"; return;
      }
      /*  Take a vertex from the stack, print it, and
          decrease the in-degree of all its immediate
          successors by 1. As soon as the updated
          in-degree of a successor is zero, place it
          on the stack and dispose of the (edge) node
          in the adjacency list.
      */
      j = start; start = a[j].count;
      J = min + j;
      cout << J << " ";
      for (;;)
      {  p = a[j].link;
         if (p == 0) break;
         k = p->num;
         a[k].count--;
         if (a[k].count == 0)
         {  a[k].count = start; start = k;
         }
         a[j].link = p->next;
         delete p;
         // In the adjacency list we have disposed of a node
         // that we don't need any longer.
      }
   }
   cout << endl;
}

int main()
{  graph g;
   char fname[50];
   cout << "Input file: ";
   cin >> setw(50) >> fname;
   g.SetRange(fname); // Find range of vertex numbers
   g.build(fname);    // Build adjacency lists for graph
   g.StartNodes();    // Form linked list of start nodes
   g.topol();         // Print topological order
   return 0;
}

