// Copyright(c) 1996 Leendert Ammeraal. All rights reserved.
// This program text occurs in Chapter 10 of
//
//    Ammeraal, L. (1996) Algorithms and Data Structures in C++,
//       Chichester: John Wiley.

/* dynpro: An application of dynamic programming.
      With a given integer s and a set of positive integers,
      we try to write s as a sum of as few terms as
      possible, each term belonging to the given set.
*/
#include <iostream.h>
#include <limits.h>
#include <stdlib.h>

class dynpro {
public:
   dynpro();
   ~dynpro()
   {  delete[]d; delete[]F;
      delete[]ref; delete[]val;
   }
   void solve(), print();
private:
   int s, n, *d, *F, *ref, *val;
   void product(int m, int v)
   {  if (m > 1) cout << m << " x ";
      cout << v;
   }
};

dynpro::dynpro()
{  cout << "Enter the desired sum: ";
   long sLong;
   cin >> sLong;
   if (sLong < 0 || sLong > INT_MAX)
   {  cout << "Invalid value\n"; exit(1);
   }  
   s = int(sLong);
   F = new int[s + 1];
   ref = new int[s + 1];
   val = new int[s + 1];
   cout << "Enter n, followed by n integers:\n";
   cin >> n;
   d = new int[n];
   for (int i=0; i<n; i++) cin >> d[i];
}

void dynpro::solve()
{  int x, u, u1, i, i1, F1;
   F[0] = 0;
   for (x=1; x<=s; x++)
   {  F1 = INT_MAX;
      for (i=0; i<n; i++)
      {  u = x - d[i];
         if (u >= 0 && F[u] < F1)
         {  F1 = F[u]; i1 = i; u1 = u;
         }
      }
      if (F1 < INT_MAX)
      {  F[x] = F1 + 1;
         ref[x] = u1;
         val[x] = d[i1];
      }  else F[x] = INT_MAX;
   }
}

void dynpro::print()
{  if (F[s] == INT_MAX)
      cout << "No solution.\n";
   else
   {  cout << "Solution:\n";
      cout << s << " = ";
      int k = s, v = val[s], m = 1;
      for (int j=2; j<=F[s]; j++)
      {  k = ref[k];
         if (val[k] == v) m++; else
         {  product(m, v); cout << " + ";
            v = val[k]; m = 1;
         }
      }
      product(m, v); cout << endl;
   }
}

int main()
{  dynpro DP;
   DP.solve();
   DP.print();
   return 0;
}
