// Copyright(c) 1996 Leendert Ammeraal. All rights reserved.
// This program text occurs in Chapter 10 of
//
//    Ammeraal, L. (1996) Algorithms and Data Structures in C++,
//       Chichester: John Wiley.

// nextperm: Find the next permutation
#include <iostream.h>

class perm {
public:
   perm();
   ~perm(){delete[]r;}
   void demo();
private:
   int *r, n;
   void NextPerm(void);
};

perm::perm()
{  cout << "Iterative permutation generator"
        << " for 1, 2, ..., n.\n"
        << "Enter n: "; cin >> n;
   r = new int[n + 1];
   for (int i=1; i<=n; i++) r[i] = i;
}

void perm::demo()
{  int code, i, j=0;
   for (;;)
   {  cout << "Enter 1 to display the next permutation"
           << " or 0 to stop: ";
      cin >> code;
      if (code == 0) return;
      cout << "Permutation " << ++j << ": ";
      for (i=1; i<=n; i++) cout << r[i] << " ";
      cout << endl;
      NextPerm();
   }
}

void perm::NextPerm()
{  /* This function replaces r[1], ..., r[n],
      containing a permutation of 1, ..., n,
      with the next permutation.
   */
   int i, k, tmp, left, right;
   // Find longest decreasing subsequence
   // r[k+1], ..., r[n] on the right:
   k = n - 1;
   while (k > 0 && r[k] > r[k+1]) k--;
   left = k + 1; right = n;
   // Convert that subsequence into an
   // increasing one:
   while (left < right)
   { tmp = r[left]; r[left++] = r[right]; r[right--] = tmp;
   }
   if (k == 0) return; // No element to the left of r[k+1]
   /* The subsequence mentioned above is increasing.
      Out of this subsequence, select the smallest
      element r[i] that is greater than r[k]. Swap r[i] and
      r[k]:
   */
   i = k+1;
   while (r[i] < r[k]) i++;
   tmp = r[i]; r[i] = r[k]; r[k] = tmp;
}

int main()
{  perm P;
   P.demo();
   return 0;
}

