// Copyright(c) 1996 Leendert Ammeraal. All rights reserved.
// This program text occurs in Chapter 10 of
//
//    Ammeraal, L. (1996) Algorithms and Data Structures in C++,
//       Chichester: John Wiley.

// perm: Generation of permutations in natural order
#include <iostream.h>

class permutations {
public:
   permutations();
   ~permutations(){delete[]r;}
   void print(){cout << "Permutations:\n"; permut(1);}
private:
   int *r, n;
   void permut(int k);
};

permutations::permutations()
{  cout << "Enter n: "; cin >> n;
   r = new int[n + 1];
   for (int i=1; i<=n; i++) r[i] = i;
}

void permutations::permut(int k)
/* The call permut(k) prints a group of (n-k+1)! lines, each
   with the n integers r[1], ..., r[k-1], r[k], ..., r[n] in
   which r[1], ..., r[k-1] do not change but in which
   r[k], ..., r[n] assume all permutations of these n-k+1
   elements on the right. It follows that the call permut(1)
   prints all n! permutations of r[1], ..., r[n]. The
   permutations are printed in ascending order.
*/
{  int i, j, tmp;
   if (k <= n)
   for (i=k; i<=n; i++)
   {  /* For each i, element r[i] is moved to
         position k and the (old) elements
         r[k], ..., r[i-1] are shifted one
         position to the right:
      */
      tmp = r[i];
      for (j=i; j>k; j--) r[j] = r[j - 1];
      r[k] = tmp;
      // Without altering r[k], a recursive call
      // is applied to r[k+1], ..., r[n]:
      permut(k+1);
      // Restore old situation:
      for (j=k; j<i; j++) r[j] = r[j+1];
      r[i] = tmp;
   }  else
   {  // If k == n, the group consists of only one
      // permutation, which is now printed:
      for (i=1; i<=n; i++) cout << r[i] << " ";
      cout << endl;
   }
}

int main()
{  permutations P;
   P.print();
   return 0;
}

