// Copyright(c) 1996 Leendert Ammeraal. All rights reserved.
// This program text occurs in Chapter 11 of
//
//    Ammeraal, L. (1996) Algorithms and Data Structures in C++,
//       Chichester: John Wiley.

/* compiler:
   This program has been derived from postfix, discussed
   in Section 11.3. It reads a VSL program, and translates
   it into the VSL object program object.c. The latter is
   a C program with the structure of an assembly-language
   program. After compiling object.c by a standard C
   compiler, it is to be linked together with the compiled
   version of rts, see Section 11.5.
*/
#include <fstream.h>
#include <iomanip.h>
#include <stdlib.h>
#include <ctype.h>

class compiler {
public:
   void prepare();
   void compile();
   void show();
private:
   void factor();
   void term();
   void expression();
   void error(const char* s)
   {  cout << "\nError: " << s << endl; exit(1);
   }
   char next();
   int NextIs(char chGiven);
   int isnum(char ch){return ch == '-' || isdigit(ch);}
   int result;
   char buf;
   ifstream ifile;
   fstream ofile;
};

void compiler::prepare()
{  char fname[51];
   cout << "This program reads an input file, containing\n";
   cout << "a VSL program, such as, for example, {3-2*5}\n";
   cout << "The resulting object module will be written\n"
           "to the file object.c.\n";
   cout << "\nName of input file: ";
   cin >> setw(51) >> fname;
   ifile.open(fname, ios::in | ios::nocreate);
   if (ifile.fail()) error("Cannot open input file.");
   ofile.open("object.c", ios::out | ios::in | ios::trunc);
   if (ofile.fail()) error("Cannot open output file.");
   cout << "VSL program read from file:\n\n";
   buf = '\0';
}

void compiler::compile()
{  if (!NextIs('{')) error("Open brace ({) expected.\n");
   ofile <<
   "int main(void)\n"
   "{  void push(int i), add(void), subtract(void),\n"
   "        multiply(void), PrintResult(void);\n";
   expression();
   ofile << "   PrintResult();\n"
            "   return 0;\n}\n";
   if (!NextIs('}')) error("Close brace (}) expected.\n");
}

void compiler::show() // Copy output file to screen:
{  char ch;
   cout << "\n\nContents of output file object.c:\n\n";
   ofile.seekg(0, ios::beg);
   for (;;)
   {  ofile.get(ch);
      if (ofile.fail()) break;
      cout << ch;
   }
}

char compiler::next()
{  char ch;
   do
   {  if (buf) {ch = buf; buf = 0;} else
      {  ifile.get(ch);
         if (ifile.fail()) error("Premature end of file?");
         cout << ch;
      }
   }  while (isspace(ch));
   return ch;
}

int compiler::NextIs(char chGiven)
{  char ch = next();
   if (ch != chGiven) buf = ch;
   return ch == chGiven;
}

void compiler::factor()
{  char ch = next();
   if (isdigit(ch))
      ofile << "   push(" << ch << ");\n"; else
   if (ch == '(')
   {  expression();
      if (!NextIs(')')) error("')' expected");
   }  else error("Digit or '(' expected");
}

void compiler::term()
{  factor();
   while (NextIs('*'))
   {  factor(); ofile << "   multiply();\n";
   }
}

void compiler::expression()
{  term();
   for ( ; ; )
   {  if (NextIs('+'))
      {  term();
         ofile << "   add();\n";
      } else
      if (NextIs('-'))
      {  term();
         ofile << "   subtract();\n";
      }  else break;
   }
}

int main()
{  compiler C;
   C.prepare();
   C.compile();
   C.show();
   return 0;
}

