// Copyright(c) 1996 Leendert Ammeraal. All rights reserved.
// This program text occurs in Chapter 11 of
//
//    Ammeraal, L. (1996) Algorithms and Data Structures in C++,
//       Chichester: John Wiley.

// interprt: A source-text interpreter for vsl

#include <fstream.h>
#include <iomanip.h>
#include <stdlib.h>
#include <ctype.h>

class interpreter {
public:
   void prepare();
   void interpret();
   void output()
   {  cout << "\n\nComputed value: " << result << endl;
   }
private:
   int factor();
   int term();
   int expression();
   void error(const char *s)
   {  cout << "\nError: " << s << endl; exit(1);
   }
   char next();
   int NextIs(char chGiven);
   int isnum(char ch){return ch == '-' || isdigit(ch);}
   int result;
   char buf;
   ifstream file;
};

void interpreter::prepare()
{  char fname[51];
   cout << "This program reads an input file, containing\n";
   cout << "a VSL program, such as, for example, {3-2*5}\n";
   cout << "\nName of input file: ";
   cin >> setw(51) >> fname;
   file.open(fname, ios::in | ios::nocreate);
   if (file.fail()) error("Cannot open input file.");
   cout << "VSL program read from file:\n\n";
   buf = '\0';
}

void interpreter::interpret()
{  if (!NextIs('{')) error("Open brace ({) expected.\n");
   result = expression();
   if (!NextIs('}')) error("Close brace (}) expected.\n");
}

char interpreter::next()
{  char ch;
   do
   {  if (buf) {ch = buf; buf = 0;} else
      {  file.get(ch);
         if (file.fail()) error("Premature end of file?");
         cout << ch;
      }
   }  while (isspace(ch));
   return ch;
}

int interpreter::NextIs(char chGiven)
{  char ch = next();
   if (ch != chGiven) buf = ch;
   return ch == chGiven;
}

int interpreter::factor()
{  int x;
   char ch=next();
   if (isdigit(ch)) x = ch - '0'; else
   if (ch == '(')
   {  x = expression();
      if (!NextIs(')')) error("')' expected");
   }  else error("Digit or '(' expected");
   return x;
}

int interpreter::term()
{  int x;
   x = factor();
   while (NextIs('*')) x *= factor();
   return x;
}

int interpreter::expression()
{  int x = term();
   for ( ; ; )
   {  if (NextIs('+')) x += term(); else
      if (NextIs('-')) x -= term(); else break;
   }
   return x;
}

int main()
{  interpreter I;
   I.prepare();
   I.interpret();
   I.output();
   return 0;
}
