// Copyright(c) 1996 Leendert Ammeraal. All rights reserved.
// This program text occurs in Chapter 11 of
//
//    Ammeraal, L. (1996) Algorithms and Data Structures in C++,
//       Chichester: John Wiley.

/* postfix: Conversion from infix to postfix; the input
      consists of a VSL program, which has the form

         { expression }

       The given expression is converted to postfix, and
       written to another file. The names of both files
       are to be supplied by the user.
*/

#include <fstream.h>
#include <iomanip.h>
#include <stdlib.h>
#include <ctype.h>

class InfixToPostfix {
public:
   void prepare();
   void convert();
   void show();
private:
   void factor();
   void term();
   void expression();
   void error(const char* s)
   {  cout << "\nError: " << s << endl; exit(1);
   }
   char next();
   int NextIs(char chGiven);
   int isnum(char ch){return ch == '-' || isdigit(ch);}
   int result;
   char buf;
   ifstream ifile;
   fstream ofile;
};

void InfixToPostfix::prepare()
{  char fname[51];
   cout << "This program reads an input file, containing\n";
   cout << "a VSL program, such as, for example, {3-2*5}\n";

   cout << "\nName of input file: ";
   cin >> setw(51) >> fname;
   ifile.open(fname, ios::in | ios::nocreate);
   if (ifile.fail()) error("Cannot open input file.");

   cout << "\nName of output file: ";
   cin >> setw(51) >> fname;
   ofile.open(fname, ios::out | ios::in | ios::trunc);
   if (ofile.fail()) error("Cannot open output file.");

   cout << "VSL program read from file:\n\n";
   buf = '\0';
}

void InfixToPostfix::convert()
{  if (!NextIs('{')) error("Open brace ({) expected.\n");
   expression();
   if (!NextIs('}')) error("Close brace (}) expected.\n");
}

void InfixToPostfix::show() // Copy output file to screen:
{  char ch;
   cout << "\n\nContents of output file (postfix):\n";
   ofile.seekg(0, ios::beg);
   for (;;)
   {  ofile >> ch;
      if (ofile.fail()) break;
      cout << ch;
   }
}

char InfixToPostfix::next()
{  char ch;
   do
   {  if (buf) {ch = buf; buf = 0;} else
      {  ifile.get(ch);
         if (ifile.fail()) error("Premature end of file?");
         cout << ch;
      }
   }  while (isspace(ch));
   return ch;
}

int InfixToPostfix::NextIs(char chGiven)
{  char ch = next();
   if (ch != chGiven) buf = ch;
   return ch == chGiven;
}

void InfixToPostfix::factor()
{  char ch = next();
   if (isdigit(ch)) ofile << ch; else
   if (ch == '(')
   {  expression();
      if (!NextIs(')')) error("')' expected");
   }  else error("Digit or '(' expected");
}

void InfixToPostfix::term()
{  factor();
   while (NextIs('*'))
   {  factor(); ofile << '*';
   }
}

void InfixToPostfix::expression()
{  term();
   for ( ; ; )
   {  if (NextIs('+')) {term(); ofile << '+';} else
      if (NextIs('-')) {term(); ofile << '-';} else
         break;
   }
}

int main()
{  InfixToPostfix IP;
   IP.prepare();
   IP.convert();
   IP.show();
   return 0;
}

