// Copyright(c) 1996 Leendert Ammeraal. All rights reserved.
// This program text occurs in Chapter 11 of
//
//    Ammeraal, L. (1996) Algorithms and Data Structures in C++,
//       Chichester: John Wiley.

/* postint:
     A postfix interpreter, which interprets the
     postfix expression produced by program 'postfix'.
*/
#include <fstream.h>
#include <iomanip.h>
#include <ctype.h>
#include <stdlib.h>

class PostfixInterpreter {
public:
   void prepare();
   void evaluate();
   void output()
   {  cout << "\nComputed result: " << s[0] << endl;
   }
private:
   enum {StackSize = 30};
   int t, s[StackSize];
   ifstream file;
   void push(int i)
   {  if (t == StackSize) error("StackSize too small");
      s[t++] = i;
   }
   void add(){t--; s[t-1] += s[t];}
   void subtract(){t--; s[t-1] -= s[t];}
   void multiply(){t--; s[t-1] *= s[t];}
   void error(const char *str)
   {  cout << "\nError: " << str << endl; exit(1);
   }
};

void PostfixInterpreter::prepare()
{  char fname[51];
   cout << "Name of input file (postfix): ";
   cin >> setw(51) >> fname;
   file.open(fname, ios::in | ios::nocreate);
   if (file.fail()) error("Cannot open input file.");
   t = 0;
}

void PostfixInterpreter::evaluate()
{  char ch;
   cout << "The given postfix expression is\n";
   for (;;)
   {  file >> ch;
      if (file.fail()) break;
      cout << ch;
      if (isdigit(ch)) push(ch - '0'); else
      if (ch == '+') add(); else
      if (ch == '-') subtract(); else
      if (ch == '*') multiply(); else break;
   }
   if (t != 1) error("incorrect postfix expression.");
}

int main()
{  PostfixInterpreter P;
   P.prepare();
   P.evaluate();
   P.output();
   return 0;
}
